/*
 *   Xfrisk - The classic board game for X
 *   Copyright (C) 1993-1999 Elan Feingold (elan@aetherworks.com)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   $Id: aiClient.h,v 1.8 2003/05/13 23:32:56 sketch Exp $
 */

#ifndef AICLIENT__
#define AICLIENT_H

#include "types.h"

/* Private stuff */
#define DefineSpecies( callback, name, author, version, desc )      \
    void *( * __AI_Callback )( void *, Int32, void * ) = callback;  \
    CString __strName = name;                                       \
    CString __strAuthor = author;                                   \
    CString __strVersion = version;                                 \
    CString __strDesc = desc;

/* Commands sent to the callback */
typedef enum
    {
    AI_INIT_ONCE        = 0,
    AI_INIT_GAME        = 1,
    AI_INIT_TURN        = 2,
    AI_FORTIFY          = 3,    /* (Number of armies) */
    AI_PLACE            = 4,
    AI_ATTACK           = 5,
    AI_MOVE             = 6,
    AI_EXCHANGE_CARDS   = 7,
    AI_SERVER_MESSAGE   = 8,    /* (Text of the message) */
    AI_MESSAGE          = 9,    /* (Text of the message) */
    AI_MOVE_MANUAL      = 10    /* (Number of armies) */
    } AI_CallbackCommands;

/* Actions computer players can perform -- PUBLIC INTERFACE */
Flag AI_Place( Int32 iCountry, Int32 iNumArmies );

Flag AI_Attack( Int32 iSrcCountry,
                Int32 iDstCountry, 
                Int32 iAttackMode,
                Int32 iDiceMode,
                Int32 iMoveMode );

Flag AI_Move( Int32 iSrcCountry,
              Int32 iDstCountry,
              Int32 iNumArmies );

Flag AI_ExchangeCards( Int32 *piCards );
Flag AI_SendMessage( Int32 iMessDest, CString strMessage );
Flag AI_EndTurn( void );

/* Attack modes */
typedef enum
    {
    ATTACK_ONCE     = 0,
    ATTACK_DOORDIE  = 1
    } AI_AttackModes;

/* Dice modes */
typedef enum
    {
    DICE_MAXIMUM    = 0,
    DICE_ONE        = 1,
    DICE_TWO        = 2,
    DICE_THREE      = 3
    } AI_DiceModes;

/* What to do with armies after taking a country.  If you wish to move
 * the maximum amount of armies into the country you might occupy with
 * the attack, pass ARMIES_MOVE_MAX.  If you want the minimum number
 * of armies moved, pass ARMIES_MOVE_MIN.  If you want finer control,
 * pass in ARMIES_MOVE_MANUAL, and your callback will be called with
 * the command AI_MOVE_MANUAL.
 */

typedef enum
    {
    ARMIES_MOVE_MAX     = 0,
    ARMIES_MOVE_MIN     = 1,
    ARMIES_MOVE_MANUAL  = 2
    } AI_MoveModes;

#endif /* AICLIENT_H */

/* End of file */
