/*
 *   Xfrisk - The classic board game for X
 *   Copyright (C) 1993-1999 Elan Feingold (elan@aetherworks.com)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   $Id: clientMain.c,v 1.24 2003/07/14 10:37:38 sketch Exp $
 */

/** \file
 * The client's main()
 *
 */

static const char
    rcsid[] = "@(#)$Id: clientMain.c,v 1.24 2003/07/14 10:37:38 sketch Exp $";

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <unistd.h>

#include "cards.h"
#include "network.h"
#include "callbacks.h"
#include "client.h"
#include "colormap.h"
#include "riskgame.h"
#include "dice.h"
#include "help.h"
#include "gui-func.h"
#include "debug.h"

/**
 * Client main function.
 */
int main( int argc, char **argv ) 
    {
    FILE   *hLogFile = NULL;
    char   *progname = argv[ 0 ];
    int    ch;
    const char *logfile = "xfrisk.log";

    /* Check args */
    while ( ( ch = getopt( argc, argv, "hl:v" ) ) != -1 )
        {
        switch ( ch )
            {
            case 'v':
                CLNT_Version( progname );
                break;

            case 'l':
                logfile = optarg;
                break;

            case 'h':
            default:
                CLNT_Usage( progname );
            }
        }

    argc -= optind;
    argv += optind;

    if ( !argc )
        {
        CLNT_Usage( progname );
        }

    /* Open debugging log file */
    if ( logfile )
        {
        if ( ( hLogFile = fopen( logfile, "a" ) ) == NULL )
            {
            printf( "Could not open \"%s\" for writing, exiting...",
                    logfile );
            exit( -1 );
            }
        }

    /* Initialize everything */
    /* Callbacks */
    RISK_InitObject( CBK_Replicate,
                     CLNT_PreViewVector,
                     CLNT_PostViewVector,
                     CLNT_RecoverFailure,
                     hLogFile );

    CLNT_Init( argc, argv );
    GUI_Setup( argc, argv );

    COLOR_Init( );

    GUI_AddCallbacks( CLIENTS_GetCommLinkOfClient( CLNT_GetThisClientID( ) ) );

    DICE_Init( );
    HELP_Init( HELPFILE );
    CARDS_Init( );

    /* And we're off... */
    GUI_Start( );

    return 0;
    }

/* EOF */
