/*
 *   Xfrisk - The classic board game for X
 *   Copyright (C) 1993-1999 Elan Feingold (elan@aetherworks.com)
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   $Id: serverMain.c,v 1.15 2003/07/14 10:37:38 sketch Exp $
 */

/** \file
 * Server startup.
 */

static const char
    rcsid[] = "@(#)$Id: serverMain.c,v 1.15 2003/07/14 10:37:38 sketch Exp $";

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include "network.h"
#include "server.h"
#include "riskgame.h"
#include "debug.h"


/**
 * Server startup function.
 *
 * \b History:
 * \arg 08.16.94  ESF  Created.
 */
int main( int argc, char **argv )
    {
    FILE   *hLogFile = NULL; 
    char   *progname = argv[ 0 ];
    int    ch;
    const char *logfile = "xfriskserver.log";

    /* Check args */
    while ( ( ch = getopt( argc, argv, "hl:v" ) ) != -1 )
        {
        switch ( ch )
            {
            case 'v':
                SRV_Version( progname );
                break;

            case 'l':
                logfile = optarg;
                break;

            case 'h':
            default:
                SRV_Usage( progname );
            }
        }

    argc -= optind;
    argv += optind;

    if ( argc )
        {
        SRV_Usage( progname );
        }

    /* Open debugging log file */
    if ( logfile )
        {
        if ( ( hLogFile = fopen( logfile, "a" ) ) == NULL )
            {
            printf( "Could not open \"%s\" for writing, exiting...",
                    logfile );
            exit( -1 );
            }
        }

    /* Initialize the distributed object */
    RISK_InitObject( SRV_Replicate, NULL, NULL, SRV_LogFailure, hLogFile );

    /* Initialize the server and start things going */
    SRV_Init( );
    SRV_PlayGame( );

    return( 0 );
    }

/**
 * Warn of erroneous arguments (we shouldn't have any) and exit.
 */
void SRV_Usage( const char *progname )
    {
    fprintf( stderr, "Usage: %s [-l <logfile>] [-v]\n", progname );
    exit( -1 );
    }

/**
 * Print server version and exit.
 */
void SRV_Version( const char *progname )
    {
    fprintf( stderr, "%s version: %s\n", progname, VERSION );
    exit( -1 );
    }

/* End of file */
