/*
 *   Xfrisk - The classic board game for X
 *   Copyright( C ) 1993-1999 Elan Feingold( elan@aetherworks.com )
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   ( at your option ) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   $Id: viewStats.c,v 1.23 2003/09/03 21:51:35 tony Exp $
 */

static const char
    rcsid[] = "@(#)$Id: viewStats.c,v 1.23 2003/09/03 21:51:35 tony Exp $";

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <X11/X.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Viewport.h>
#include <X11/Xaw/List.h>

#include <assert.h>

#include "network.h"
#include "viewStats.h"
#include "riskgame.h"
#include "callbacks.h"
#include "gui-vars.h"
#include "utils.h"
#include "types.h"
#include "debug.h"

/* Defines */
#define GRAPH_WIDTH     551
#define GRAPH_HEIGHT    150

/* dX for graph 3 ok-ish */
#define GRAPH_STEP      3

/* Private globals */
static Widget   wStatShell;
static Widget   wStatForm;
static Widget   wStatNumberForm;
static Widget   wStatGraphForm;
static Widget   wStatColorLabel;
static Widget   wStatNameLabel;
static Widget   wStatArmiesLabel;
static Widget   wStatCountriesLabel;
static Widget   wStatCardsLabel;
static Widget   wStatWLDLabel;
static Widget   wStatATLabel;
static Widget   wStatViewport;
static Widget   wStatPlayerForm;
static Widget   wPlayerStatForm[ MAX_PLAYERS ];
static Widget   wStatPlayerColor[ MAX_PLAYERS ];
static Widget   wStatPlayerName[ MAX_PLAYERS ];
static Widget   wStatPlayerArmies[ MAX_PLAYERS ];
static Widget   wStatPlayerCountries[ MAX_PLAYERS ];
static Widget   wStatPlayerCards[ MAX_PLAYERS ];
static Widget   wStatPlayerWLD[ MAX_PLAYERS ];
static Widget   wStatPlayerAT[ MAX_PLAYERS ];
static Widget   wStatGraphLabel;
static Widget   wStatGraph;
static Widget   wStatCloseButton;

static Int32    piSlotToPlayer[ MAX_PLAYERS ];
static Int32    piPlayerWin[ MAX_PLAYERS ];
static Int32    piPlayerLose[ MAX_PLAYERS ];
static Int32    piPlayerDraw[ MAX_PLAYERS ];
static Int32    piTotalArmies[ MAX_PLAYERS ];
static Int32    piLastValue[ MAX_PLAYERS ];
static Flag     fDialogIsUp = FALSE;
static Int32    fFirstTime  = TRUE;
static Int32    iXPos       = 0;
static Int32    iBackgroundColor;
static Pixmap   pixmap;

/* Private functions */
Int32 STAT_SlotToPlayer( Int32 iSlot );

void  STAT_InitThings( void );

void  STAT_UpdateGraph( void );
void  STAT_UpdateArmies( Int32 iPlayer, Int32 iArmies );
void  STAT_UpdateCountries( Int32 iPlayer, Int32 iCountries );
void  STAT_UpdateCards( Int32 iPlayer, Int32 iCards );
void  STAT_UpdateWinLoseDraw( Int32 iPlayer );
void  STAT_UpdateArmiesPerTurn( Int32 iPlayer );

void  STAT_PlayerCreated( Int32 iPlayer );
void  STAT_PlayerDestroyed( Int32 iPlayer );


/************************************************************************
 *  FUNCTION: STAT_AddPlayerStats
 *  HISTORY:
 *     12.25.95  TdH  Created.
 *  PURPOSE: Add info lines about players
 *  NOTES:
 ************************************************************************/

void STAT_AddPlayerStats( void )
    {
    Int32   i;

    /* Each player */
    for ( i = 0; i != MAX_PLAYERS; i++ )
        {
        wPlayerStatForm[ i ] = XtVaCreateManagedWidget( "wShowPlayerForm",
                                                        formWidgetClass,
                                                        wStatPlayerForm,
                                                        NULL );
        if ( i )
            {
            XtVaSetValues( wPlayerStatForm[ i ],
                           XtNfromVert,
                           wPlayerStatForm[ i - 1  ],
                           NULL );
            }
        wStatPlayerColor[ i ] = XtVaCreateManagedWidget( "wStatPlayerColor",
                                                         labelWidgetClass,
                                                         wPlayerStatForm[ i ],
                                                         XtNresize,
                                                         False,
                                                         NULL );

        wStatPlayerName[ i ] = XtVaCreateManagedWidget( "wStatPlayerName",
                                                        labelWidgetClass,
                                                        wPlayerStatForm[ i ],
                                                        XtNfromHoriz,
                                                        wStatPlayerColor[ i ],
                                                        XtNresize,
                                                        False,
                                                       NULL );

        wStatPlayerArmies[ i ] = XtVaCreateManagedWidget( "wStatPlayerArmies",
                                                          labelWidgetClass,
                                                          wPlayerStatForm[ i ],
                                                          XtNfromHoriz,
                                                          wStatPlayerName[ i ],
                                                          XtNresize,
                                                          False,
                                                          NULL );

        wStatPlayerCountries[ i ] = XtVaCreateManagedWidget( "wStatPlayerCountries",
                                                             labelWidgetClass,
                                                             wPlayerStatForm[ i ],
                                                             XtNfromHoriz,
                                                             wStatPlayerArmies[ i ],
                                                             XtNresize,
                                                             False,
                                                             NULL );

        wStatPlayerCards[ i ] = XtVaCreateManagedWidget( "wStatPlayerCards",
                                                         labelWidgetClass,
                                                         wPlayerStatForm[ i ],
                                                         XtNfromHoriz,
                                                         wStatPlayerCountries[ i ],
                                                         XtNresize,
                                                         False,
                                                         NULL );

        wStatPlayerWLD[ i ] = XtVaCreateManagedWidget( "wStatPlayerWLD",
                                                       labelWidgetClass,
                                                       wPlayerStatForm[ i ],
                                                       XtNfromHoriz,
                                                       wStatPlayerCards[ i ],
                                                       XtNresize,
                                                       False,
                                                      NULL );

        wStatPlayerAT[ i ] = XtVaCreateManagedWidget( "wStatPlayerAT",
                                                      labelWidgetClass,
                                                      wPlayerStatForm[ i ],
                                                      XtNfromHoriz,
                                                      wStatPlayerWLD[ i ],
                                                      XtNresize,
                                                      False,
                                                      NULL );

        }
    }

/************************************************************************
 *  FUNCTION:
 *  HISTORY:
 *     02.17.95  ESF  Created.
 *  PURPOSE:
 *  NOTES:
 ************************************************************************/
void STAT_BuildDialog( void )
    {
    Int32   i;

    wStatShell = XtCreatePopupShell( "wStatShell",
                                     topLevelShellWidgetClass,
                                     wToplevel,
                                     pVisualArgs,
                                     iVisualCount );

    /* The forms */
    wStatForm = XtCreateManagedWidget( "wStatForm",
                                       formWidgetClass,
                                       wStatShell,
                                       NULL,
                                       0 );

    wStatNumberForm = XtCreateManagedWidget( "wStatNumberForm",
                                             formWidgetClass,
                                             wStatForm,
                                             NULL,
                                             0 );

    wStatGraphForm = XtCreateManagedWidget( "wStatGraphForm",
                                            formWidgetClass,
                                            wStatForm,
                                            NULL,
                                            0 );

    wStatCloseButton = XtCreateManagedWidget( "wStatCloseButton",
                                              commandWidgetClass,
                                              wStatForm,
                                              NULL,
                                              0 );

    XtAddCallback( wStatCloseButton,
                   XtNcallback,
                   ( XtCallbackProc ) STAT_Close,
                   NULL );

    /* The labels */
    wStatColorLabel = XtCreateManagedWidget( "wStatColorLabel",
                                             labelWidgetClass,
                                             wStatNumberForm,
                                             NULL,
                                             0 );

    wStatNameLabel = XtCreateManagedWidget( "wStatNameLabel",
                                            labelWidgetClass,
                                            wStatNumberForm,
                                            NULL,
                                            0 );

    wStatArmiesLabel = XtCreateManagedWidget( "wStatArmiesLabel",
                                              labelWidgetClass,
                                              wStatNumberForm,
                                              NULL,
                                              0 );

    wStatCountriesLabel = XtCreateManagedWidget( "wStatCountriesLabel",
                                                 labelWidgetClass,
                                                 wStatNumberForm,
                                                 NULL,
                                                 0 );

    wStatCardsLabel = XtCreateManagedWidget( "wStatCardsLabel",
                                             labelWidgetClass,
                                             wStatNumberForm,
                                             NULL,
                                             0 );

    wStatWLDLabel = XtCreateManagedWidget( "wStatWLDLabel",
                                           labelWidgetClass,
                                           wStatNumberForm,
                                           NULL,
                                          0 );

    wStatATLabel = XtCreateManagedWidget( "wStatATLabel",
                                          labelWidgetClass,
                                          wStatNumberForm,
                                          NULL,
                                          0 );

    /* The statistics "spreadsheet" */
    wStatViewport = XtCreateManagedWidget( "wStatViewport",
                                           viewportWidgetClass,
                                           wStatNumberForm,
                                           NULL,
                                           0 );

    wStatPlayerForm = XtCreateManagedWidget( "wStatPlayerForm",
                                             boxWidgetClass,
                                             wStatViewport,
                                             NULL,
                                             0 );

    STAT_AddPlayerStats( ); /* info about each player */

    /* The statistics graph */
    wStatGraphLabel = XtVaCreateManagedWidget( "wStatGraphLabel",
                                               labelWidgetClass,
                                               wStatGraphForm,
                                               NULL );

    wStatGraph = XtVaCreateManagedWidget( "wStatGraph",
                                          labelWidgetClass,
                                          wStatGraphForm,
                                          NULL );

    /* Create the graph pixmap */
    pixmap = XCreatePixmap( hDisplay,
                            RootWindowOfScreen( XtScreen( wStatGraph ) ),
                            GRAPH_WIDTH,
                            GRAPH_HEIGHT,
                            DefaultDepth( hDisplay,
                                          DefaultScreen( hDisplay ) ) );

    /* Set it to be the background pixmap of the widget
     */
    XtVaSetValues( wStatGraph,
                   XtNbitmap,
                   pixmap,
                   NULL );

    STAT_InitThings( );

    /* In addition, reset the mapping -- this happens but once
     * this is why it's not in STAT_InitThings( )*/
    for ( i = 0; i != MAX_PLAYERS; i++ )
        {
        piSlotToPlayer[ i ] = -1;
        }
    }


/************************************************************************
 *  FUNCTION:
 *  HISTORY:
 *     04.01.95  ESF  Created.
 *  PURPOSE:
 *  NOTES:
 ************************************************************************/
void STAT_InitThings( void )
    {
    Int32   i;

    /* Init the data & slots */
    for ( i = 0; i != MAX_PLAYERS; i++ )
        {
        piPlayerWin[ i ]    = 0;
        piPlayerLose[ i ]   = 0;
        piPlayerDraw[ i ]   = 0;
        piTotalArmies[ i ]  = 0;
        if ( fDialogIsUp )
            {
            STAT_UpdateWinLoseDraw( i );
            STAT_UpdateArmies( i, piTotalArmies[ i ] );
            }
        }

    /* Fill the pixmap with the color background of the ocean.
     * Tdh: and ocean color is borked. easiest fix would be a global SEACOLOR
     */
    XSetForeground( hDisplay, hGC, 0 );

    XFillRectangle( hDisplay,
                    pixmap,
                    hGC,
                    0,
                    0,
                    GRAPH_WIDTH,
                    GRAPH_HEIGHT );

    /* If the dialog is up, then blit the new stuff to it */
    if ( fDialogIsUp )
        {
        XCopyArea( hDisplay,
                   pixmap,
                   XtWindow( wStatGraph ),
                   hGC,
                   0,
                   0,
                   GRAPH_WIDTH,
                   GRAPH_HEIGHT,
                   0,
                   0 );
        }
    iXPos = 0;

    /* This will trigger the first points on the graph to be drawn */
    fFirstTime = TRUE;
    }


/************************************************************************
 *  FUNCTION:
 *  HISTORY:
 *     02.17.95  ESF  Created.
 *  PURPOSE:
 *  NOTES:
 ************************************************************************/
void STAT_PopupDialog( void )
    {
    Int32   x;
    Int32   y;
    Int32   i;
    Int32   iPlayer;

    if ( fDialogIsUp )
        {
        fDialogIsUp = FALSE;
        XtPopdown( wStatShell );
        return;
        }

    /* Center the new shell */
    UTIL_CenterShell( wStatShell, wToplevel, &x, &y );

    XtVaSetValues( wStatShell,
                   XtNallowShellResize,
                   False,
                   XtNx,
                   x,
                   XtNy,
                   y,
                   XtNborderWidth,
                   1,
                   XtNtitle,
                   "Game Statistics",
                   NULL );

    /* It's going to be up in a second. */
    fDialogIsUp = TRUE;

    for ( i = 0; i < RISK_GetNumPlayers( ); i++ )
        {
        iPlayer = RISK_GetNthPlayer( i );

        STAT_UpdateArmies( iPlayer, piTotalArmies[ iPlayer ] );
        STAT_UpdateCountries( iPlayer, RISK_GetNumCountriesOfPlayer( iPlayer ) );
        STAT_UpdateCards( iPlayer, RISK_GetNumCardsOfPlayer( iPlayer ) );
        STAT_UpdateWinLoseDraw( iPlayer );
        STAT_UpdateArmiesPerTurn( iPlayer );
        }

    /* Pop it up... */
    XtPopup( wStatShell, XtGrabNone );

    /* Get the background color for erasing slots */
    XtVaGetValues( wPlayerStatForm[ 0 ],
                   XtNborderColor,
                   &iBackgroundColor,
                   NULL );
    }


/************************************************************************
 *  FUNCTION:
 *  HISTORY:
 *     02.17.95  ESF  Created.
 *  PURPOSE:
 *  NOTES:
 ************************************************************************/
void STAT_Close( void )
    {
    STAT_PopupDialog( );
    }


/************************************************************************
 *  FUNCTION:
 *  HISTORY:
 *     02.17.95  ESF  Created.
 *     28.08.95  JC   MSG_ENDOFGAME -> MSG_ENDOFMISSION or MSG_VICTORY.
 *  PURPOSE:
 *  NOTES:
 ************************************************************************/
void STAT_Callback( const Msgs *pMess )
    {
    Int32   iPlayer;
    Int32   iCountry;
    Int32   iOldValue;
    Int32   iNewValue;
    Int32   i;
    Int32   iOtherPlayer;

    /*
     * GameBegin              --> Init. things
     * EndTurn                --> Territories graph
     * DiceRoll               --> Win / Lose / Draw
     * Object(iNumCountries)  --> Update
     * Object(iNumCards)      --> Update
     * Object(iNumArmies)     --> Update
     * PlayerCreated          --> Add player
     * PlayerDestroyed        --> Delete player
     */
    if ( pMess->msgId == MSG_STARTGAME )
        {
        /* Reset things that need resetting between games */
        STAT_InitThings( );
        }
    else if ( pMess->msgId == MSG_DICEROLL )
        {
        /* Go through the dice roll and update the
         * win / lose / draw statistics for the players.
         */

        iOtherPlayer = pMess->msgDiceRoll.iDefendingPlayer;

        for ( i = 0; i != 3; i++ )
            if ( pMess->msgDiceRoll.pAttackDice[ i ] != -1 &&
                 pMess->msgDiceRoll.pDefendDice[ i ] != -1 )
                {
                if ( pMess->msgDiceRoll.pAttackDice[ i ] > pMess->msgDiceRoll.pDefendDice[ i ] )
                    {
                    piPlayerWin[ iCurrentPlayer ]++;
                    piPlayerLose[ iOtherPlayer ]++;
                    }
                else if ( pMess->msgDiceRoll.pAttackDice[ i ] == pMess->msgDiceRoll.pDefendDice[ i ] )
                    {
                    piPlayerDraw[ iCurrentPlayer ]++;
                    piPlayerDraw[ iOtherPlayer ]++;
                    }
                else if ( pMess->msgDiceRoll.pAttackDice[ i ] < pMess->msgDiceRoll.pDefendDice[ i ] )
                    {
                    piPlayerLose[ iCurrentPlayer ]++;
                    piPlayerWin[ iOtherPlayer ]++;
                    }
                }

        /* Win / Lose / Draw */
        if ( fDialogIsUp )
            {
            STAT_UpdateWinLoseDraw( iCurrentPlayer );
            STAT_UpdateWinLoseDraw( iOtherPlayer );
            }
        }

    else if ( ( pMess->msgId == MSG_ENDOFMISSION ) || ( pMess->msgId == MSG_VICTORY ) )
        {
        /* The last position of the board, since ENDTURN won't come along. */
        STAT_UpdateGraph( );
        }

    else if ( pMess->msgId == MSG_TURNNOTIFY )
        {
        /* Draws the initial points on the graph */
        if ( fFirstTime )
            {
            fFirstTime = FALSE;
            STAT_UpdateGraph( );
            }
        }

    else if ( pMess->msgId == MSG_ENDTURN )
        {
        /* Territories graph, but only if not in fortifying mode... */
        if ( iState != STATE_FORTIFY )
            {
            STAT_UpdateGraph( );
            }
        }

    else if ( pMess->msgId == MSG_OBJINTUPDATE &&
              pMess->msgObjIntUpdate.iField == PLR_ALLOCATION &&
              pMess->msgObjIntUpdate.iNewValue == ALLOC_COMPLETE )
        {
        /* PlayerCreated */
        STAT_PlayerCreated( pMess->msgObjIntUpdate.iIndex1 );
        }

    else if ( pMess->msgId == MSG_OBJINTUPDATE &&
              pMess->msgObjIntUpdate.iField == PLR_ALLOCATION &&
              pMess->msgObjIntUpdate.iNewValue == ALLOC_NONE )
        {
        /* PlayerDestroyed */
        STAT_PlayerDestroyed( pMess->msgObjIntUpdate.iIndex1 );
        }

    else if ( pMess->msgId == MSG_OBJINTUPDATE &&
              pMess->msgObjIntUpdate.iField == PLR_NUMCOUNTRIES )
        {
        iPlayer = pMess->msgObjIntUpdate.iIndex1;

        /* Object( iNumCountries ) */
        if ( iPlayer >= 0 )
            {
            STAT_UpdateCountries( iPlayer, pMess->msgObjIntUpdate.iNewValue );
            STAT_UpdateArmiesPerTurn( iPlayer );
            }
        }

    else if ( pMess->msgId == MSG_OBJINTUPDATE &&
              pMess->msgObjIntUpdate.iField == PLR_NUMCARDS )
        {
        iPlayer = pMess->msgObjIntUpdate.iIndex1;

        /* Object( iNumCountries ) */
        if ( iPlayer >= 0 )
            {
            STAT_UpdateCards( iPlayer, pMess->msgObjIntUpdate.iNewValue );
            }
        }

    else if ( pMess->msgId == MSG_OBJINTUPDATE &&
              pMess->msgObjIntUpdate.iField == CNT_NUMARMIES )
        {
        iCountry    = pMess->msgObjIntUpdate.iIndex1;
        iPlayer     = RISK_GetOwnerOfCountry( iCountry );
        iNewValue   = pMess->msgObjIntUpdate.iNewValue;
        iOldValue   = RISK_GetNumArmiesOfCountry( iCountry );

        if ( iPlayer >= 0 )
            {
            /* Update the number of total armies the player has */
            piTotalArmies[ iPlayer ] += ( iNewValue - iOldValue );
            STAT_UpdateArmies( iPlayer, piTotalArmies[ iPlayer ] );
            STAT_UpdateArmiesPerTurn( iPlayer );
            }
        }
    }


/************************************************************************
 *  FUNCTION:
 *  HISTORY:
 *     02.19.95  ESF  Created.
 *  PURPOSE:
 *  NOTES:
 ************************************************************************/
void STAT_UpdateGraph( void )
    {
    Int32   iYPos;
    Int32   i;

    /* Assertion: iXPos is the new X position. */

    /* Check to see if we are end the right end of the graph */
    if ( iXPos == GRAPH_WIDTH )
        {
        /* Pick the two-thirds point */
        Int32 iNewStartX = GRAPH_WIDTH / 3 * 2;

        /* Scroll the graph over a third to the left */
        XCopyArea( hDisplay,
                   pixmap,
                   pixmap,
                   hGC,
                   iNewStartX / 2,
                   0,
                   GRAPH_WIDTH - iNewStartX / 2,
                   GRAPH_HEIGHT,
                   0,
                   0 );

        /* Clean up the right third now */
        XSetForeground( hDisplay,
                        hGC,
                        COLOR_QueryColor( COLOR_CountryToColor( NUM_COUNTRIES ) ) );

        XFillRectangle( hDisplay,
                        pixmap,
                        hGC,
                        iNewStartX,
                        0,
                        GRAPH_WIDTH - iNewStartX,
                        GRAPH_HEIGHT );

        iXPos = iNewStartX;
        }

    /* Assertion: iXPos is the next X place to draw the next point */

    /* Put the number of countries of each player on the graph */
    for ( i = 0; i != MAX_PLAYERS; i++ )
        {
        /* Pick the color of player i */
        XSetForeground( hDisplay,
                        hGC,
                        COLOR_QueryColor( COLOR_PlayerToColor( i ) ) );

        /* Calculate the y point */
        iYPos = ( GRAPH_HEIGHT -
                  RISK_GetNumCountriesOfPlayer( i ) *
                  GRAPH_HEIGHT /
                  NUM_COUNTRIES );

        if ( iXPos == 0 )
            {
            /* First point, just draw a point */
            XDrawPoint( hDisplay,
                        pixmap,
                        hGC,
                        iXPos,
                        iYPos );
            }
        else
            {
            /* Draw a line from the last point to the current one */
            XDrawLine( hDisplay,
                       pixmap,
                       hGC,
                       iXPos - GRAPH_STEP,
                       piLastValue[ i ],
                       iXPos,
                       iYPos );

            XDrawLine( hDisplay,
                       pixmap,
                       hGC,
                       iXPos - GRAPH_STEP,
                       piLastValue[ i ] + 1,
                       iXPos,
                       iYPos + 1 );
            }

        /* Copy portion to screen so it shows <==> the window's up */
        if ( fDialogIsUp )
            {
            XCopyArea( hDisplay,
                       pixmap,
                       XtWindow( wStatGraph ),
                       hGC,
                       iXPos - GRAPH_STEP,
                       0,
                       2,
                       GRAPH_HEIGHT,
                       iXPos - GRAPH_STEP,
                       0 );
            }

        /* Update the last y-value for the player i */
        piLastValue[ i ] = iYPos;
        }

    /* Go to the next X position */
    iXPos += GRAPH_STEP;
    }


/************************************************************************
 *  FUNCTION:
 *  HISTORY:
 *     02.19.95  ESF  Created.
 *  PURPOSE:
 *  NOTES:
 ************************************************************************/
void STAT_UpdateArmies( Int32 iPlayer, Int32 iArmies )
    {
    char    buf[ 256 ];
    Int32   iSlot = STAT_PlayerToSlot( iPlayer );

    if ( iSlot != -1 )
        {
        snprintf( buf,
                  sizeof( buf ),
                  "%d",
                  iArmies );

        XtVaSetValues( wStatPlayerArmies[ iSlot ],
                       XtNlabel,
                       buf,
                       NULL );
        }
    }


/************************************************************************
 *  FUNCTION:
 *  HISTORY:
 *     02.19.95  ESF  Created.
 *  PURPOSE:
 *  NOTES:
 ************************************************************************/
void STAT_UpdateCountries( Int32 iPlayer, Int32 iCountries )
    {
    char    buf[ 256 ];
    Int32   iSlot = STAT_PlayerToSlot( iPlayer );

    if ( iSlot != -1 )
        {
        snprintf( buf,
                  sizeof( buf ),
                  "%d",
                  iCountries );

        XtVaSetValues( wStatPlayerCountries[ iSlot ],
                       XtNlabel,
                       buf,
                       NULL );
        }
    }


/************************************************************************
 *  FUNCTION:
 *  HISTORY:
 *     02.19.95  ESF  Created.
 *  PURPOSE:
 *  NOTES:
 ************************************************************************/
void STAT_UpdateCards( Int32 iPlayer, Int32 iCards )
    {
    char    buf[ 256 ];
    Int32   iSlot = STAT_PlayerToSlot( iPlayer );

    if ( iSlot != -1 )
        {
        snprintf( buf,
                  sizeof( buf ),
                  "%d",
                  iCards );

        XtVaSetValues( wStatPlayerCards[ iSlot ],
                       XtNlabel,
                       buf,
                       NULL );
        }
    }


/************************************************************************
 *  FUNCTION:
 *  HISTORY:
 *     02.19.95  ESF  Created.
 *  PURPOSE:
 *  NOTES:
 ************************************************************************/
void STAT_UpdateWinLoseDraw( Int32 iPlayer )
    {
    char    buf[ 256 ];
    Int32   iSlot = STAT_PlayerToSlot( iPlayer );

    if ( iSlot != -1 )
        {
        snprintf( buf,
                  sizeof( buf ),
                  "%d/%d/%d",
                  piPlayerWin[ iPlayer ],
                  piPlayerLose[ iPlayer ],
                  piPlayerDraw[ iPlayer ] );

        XtVaSetValues( wStatPlayerWLD[ iSlot ],
                       XtNlabel,
                       buf,
                       NULL );
        }
    }


 /************************************************************************
  *  FUNCTION:
  *  HISTORY:
 *     17.08.03  JJK  Created.
 *  PURPOSE:
 *  NOTES:
 ************************************************************************/
void STAT_UpdateArmiesPerTurn( Int32 iPlayer )
    {
    char    buf[ 256 ];
    Int32   iSlot = STAT_PlayerToSlot( iPlayer );
       Int32       iArmiesTurn;

    if ( iSlot != -1 )
        {
                   iArmiesTurn = GAME_GetArmiesPerTurn( iPlayer );

           snprintf( buf,
                     sizeof( buf ),
                   "%d",
                   iArmiesTurn );

           XtVaSetValues( wStatPlayerAT[ iSlot ],
                        XtNlabel,
                          buf,
                        NULL );
        }
    }


/************************************************************************
 *  FUNCTION:
 *  HISTORY:
 *     02.19.95  ESF  Created.
 *  PURPOSE:
 *  NOTES:
 ************************************************************************/
void STAT_PlayerCreated( Int32 iPlayer )
  {
    Int32   iSlot;

    /* Search for open slot */
    for ( iSlot = 0; iSlot < MAX_PLAYERS; iSlot++ )
        {
        if ( piSlotToPlayer[ iSlot ] == -1 )
            {
            break;
            }
        }

    /* There MUST be a slot */
    assert( iSlot < MAX_PLAYERS );

    /* Actually dump the data to the slot */
    STAT_RenderSlot( iSlot, iPlayer );
    }


/************************************************************************
 *  FUNCTION:
 *  HISTORY:
 *     02.19.95  ESF  Created.
 *  PURPOSE:
 *  NOTES:
 ************************************************************************/
void STAT_PlayerDestroyed( Int32 iPlayer )
    {
    Int32   iSlot;
    Int32   i;

    /* Which slot is the player in? */
    iSlot = STAT_PlayerToSlot( iPlayer );

    /* Move all the other entries up */
    if ( iSlot != -1 )
        {
        for ( i = iSlot;
              ( i < MAX_PLAYERS - 1 ) &&
                  ( piSlotToPlayer[ i ] != -1 ) &&
                  ( piSlotToPlayer[ i + 1   ]!= -1 );
              i++ )
            {
            /* Move slot i + 1 to i */
            iPlayer = piSlotToPlayer[ i + 1  ];
            piSlotToPlayer[ i ] = iPlayer;

            /* Fill the destination slot with the new player */
            STAT_RenderSlot( i, iPlayer );
            }

        /* Erase the last one */
        STAT_RenderSlot( i, -1 );
        }
    else
        {
            /* Place for an error message, incoming message was corrupt
             * this happens when cancelling "add player"
             */
        assert( iSlot != -1 );
        }
    }


/************************************************************************
 *  FUNCTION:
 *  HISTORY:
 *     02.19.95  ESF  Created.
 *  PURPOSE:
 *  NOTES:
 ************************************************************************/
void STAT_RenderSlot( Int32 iSlot, Int32 iPlayer )
    {
    assert( iSlot != -1 );

    /* Either render the slot with a player or delete it */
    if ( iPlayer == -1 )
        {
        /* Reset the slot */
        piSlotToPlayer[ iSlot ] = -1;

        /* Fill in the color */
        XtVaSetValues( wStatPlayerColor[ iSlot ],
                       XtNbackground,
                       iBackgroundColor,
                       XtNborderColor,
                       iBackgroundColor,
                       NULL );

        /* The name */
        XtVaSetValues( wStatPlayerName[ iSlot ], XtNlabel, "", NULL );

        /* The armies */
        XtVaSetValues( wStatPlayerArmies[ iSlot ], XtNlabel, "", NULL );

        /* The countries */
        XtVaSetValues( wStatPlayerCountries[ iSlot ], XtNlabel, "", NULL );

        /* The cards */
        XtVaSetValues( wStatPlayerCards[ iSlot ], XtNlabel, "", NULL );

        /* The Win / Lose / Draw */
        XtVaSetValues( wStatPlayerWLD[ iSlot ], XtNlabel, "", NULL );

        /* The Armies per turn */
       XtVaSetValues( wStatPlayerAT[ iSlot ], XtNlabel, "", NULL );

        }
    else
        {
        /* Set the mapping */
        piSlotToPlayer[ iSlot ] = iPlayer;

        /* Fill in the color and change the cursor */
        XtVaSetValues( wStatPlayerColor[ iSlot ],
                       XtNbackground,
                       COLOR_QueryColor( COLOR_PlayerToColor( iPlayer ) ),
                       XtNborderColor,
                       BlackPixel( hDisplay, 0 ),
                       NULL );

        /* The name */
        XtVaSetValues( wStatPlayerName[ iSlot ],
                       XtNlabel,
                       RISK_GetNameOfPlayer( iPlayer ),
                       NULL );

        /* The countries */
        STAT_UpdateCountries( iPlayer, RISK_GetNumCountriesOfPlayer( iPlayer ) );

        /* The armies */
        STAT_UpdateArmies( iPlayer, piTotalArmies[ iPlayer ] );

        /* The cards */
        STAT_UpdateCards( iPlayer, RISK_GetNumCardsOfPlayer( iPlayer ) );

        /* The Win / Lose / Draw */
        STAT_UpdateWinLoseDraw( iPlayer );

        /* The Armies per turn */
        STAT_UpdateArmiesPerTurn( iPlayer );

        }
    }


/************************************************************************
 *  FUNCTION:
 *  HISTORY:
 *     02.19.95  ESF  Created.
 *  PURPOSE:
 *  NOTES:
 ************************************************************************/
Int32 STAT_PlayerToSlot( Int32 iPlayer )
    {
    Int32   iSlot;

    /* Which slot is the player in? */
    for ( iSlot = 0; iSlot != MAX_PLAYERS; iSlot++ )
        {
        if ( piSlotToPlayer[ iSlot ] == iPlayer )
            {
            return iSlot;
            }
        }

    return -1;
    }

/* End of file */
